@@ -1,5 +1,5 @@
 """This module is used to scrape the all of the APIs from a given source file
-and return thier name and kind.  These include classes, structs, functions, 
+and return their name and kind.  These include classes, structs, functions,
 and certain variable types.  It is not used to actually describe these elements.
 That is the job of the autodescriber.
 
@@ -35,6 +35,7 @@ import re
 import sys
 from hashlib import md5
 from pprint import pprint, pformat
+from warnings import warn
 try:
     import cPickle as pickle
 except ImportError:
@@ -45,6 +46,12 @@ try:
 except ImportError:
     pycparser = None
 
+try:
+    from . import clang
+    from .clang.cindex import CursorKind
+except ImportError:
+    clang = None
+
 from . import utils
 from . import astparsers
 
@@ -154,7 +161,8 @@ class GccxmlFinder(object):
             
 
 def gccxml_findall(filename, includes=(), defines=('XDRESS',), undefines=(),
-                   verbose=False, debug=False,  builddir='build'):
+                   extra_parser_args=(), verbose=False, debug=False, 
+                   builddir='build', language='c++', clang_includes=()):
     """Automatically finds all API elements in a file via GCC-XML.
 
     Parameters
@@ -167,19 +175,17 @@ def gccxml_findall(filename, includes=(), defines=('XDRESS',), undefines=(),
         The list of extra macro definitions to apply.
     undefines : list of str, optional
         The list of extra macro undefinitions to apply.
-    parsers : str, list, or dict, optional
-        The parser / AST to use to use for the file.  Currently 'clang', 'gccxml', 
-        and 'pycparser' are supported, though others may be implemented in the 
-        future.  If this is a string, then this parser is used.  If this is a list, 
-        this specifies the parser order to use based on availability.  If this is
-        a dictionary, it specifies the order to use parser based on language, i.e.
-        ``{'c' ['pycparser', 'gccxml'], 'c++': ['gccxml', 'pycparser']}``.
+    extra_parser_args : list of str, optional
+        Further command line arguments to pass to the parser.
     verbose : bool, optional
         Flag to diplay extra information while describing the class.
     debug : bool, optional
         Flag to enable/disable debug mode.
     builddir : str, optional
         Location of -- often temporary -- build files.
+    language : str
+        Valid language flag.
+    clang_includes : ignored
 
     Returns
     -------
@@ -195,7 +201,8 @@ def gccxml_findall(filename, includes=(), defines=('XDRESS',), undefines=(),
         # GCC-XML and/or Cygwin wants posix paths on Windows.
         filename = posixpath.join(*ntpath.split(filename))
     root = astparsers.gccxml_parse(filename, includes=includes, defines=defines,
-            undefines=undefines, verbose=verbose, debug=debug, builddir=builddir)
+            undefines=undefines, extra_parser_args=extra_parser_args, verbose=verbose, 
+            debug=debug, builddir=builddir)
     basename = filename.rsplit('.', 1)[0]
     onlyin = set([filename] + 
                  [basename + '.' + h for h in utils._hdr_exts if h.startswith('h')])
@@ -203,9 +210,68 @@ def gccxml_findall(filename, includes=(), defines=('XDRESS',), undefines=(),
     finder.visit()
     return finder.variables, finder.functions, finder.classes
 
-@astparsers.not_implemented
-def clang_findall(*args, **kwargs):
-    pass
+
+def clang_findall(filename, includes=(), defines=('XDRESS',), undefines=(),
+                  extra_parser_args=(), verbose=False, debug=False, builddir='build', 
+                  language='c++', clang_includes=()):
+    """Automatically finds all API elements in a file via clang.
+
+    Parameters
+    ----------
+    filename : str
+        The path to the file
+    includes : list of str, optional
+        The list of extra include directories to search for header files.
+    defines : list of str, optional
+        The list of extra macro definitions to apply.
+    undefines : list of str, optional
+        The list of extra macro undefinitions to apply.
+    extra_parser_args : list of str, optional
+        Further command line arguments to pass to the parser.
+    language : str
+        Valid language flag.
+    verbose : Ignored
+    debug : Ignored
+    builddir : Ignored
+    clang_includes : list of str, optional
+        clang-specific include paths.
+
+    Returns
+    -------
+    variables : list of strings
+        A list of variable names to wrap from the file.
+    functions : list of strings
+        A list of function names to wrap from the file.
+    classes : list of strings
+        A list of class names to wrap from the file.
+
+    """
+    tu = astparsers.clang_parse(filename, includes=includes, defines=defines,
+                                undefines=undefines, 
+                                extra_parser_args=extra_parser_args, verbose=verbose, 
+                                debug=debug, language=language, 
+                                clang_includes=clang_includes)
+    basename = filename.rsplit('.', 1)[0]
+    onlyin = frozenset([filename] +
+                       [basename + '.' + h for h in utils._hdr_exts if h.startswith('h')])
+    variables, functions, classes = [],[],[]
+    def visit(node):
+        kind = node.kind
+        if kind == CursorKind.NAMESPACE:
+            for kid in node.get_children():
+                visit(kid)
+        elif kind == CursorKind.ENUM_DECL:
+            variables.append(node.spelling)
+        elif kind == CursorKind.FUNCTION_DECL:
+            functions.append(node.spelling)
+        elif kind in (CursorKind.CLASS_DECL,CursorKind.STRUCT_DECL):
+            classes.append(node.spelling)
+    for node in tu.cursor.get_children():
+        file = node.extent.start.file
+        if file and file.name in onlyin:
+            visit(node)
+    return variables, functions, classes
+
 
 class PycparserFinder(astparsers.PycparserNodeVisitor):
     """Class used for discovering APIs using the pycparser AST."""
@@ -252,7 +318,7 @@ class PycparserFinder(astparsers.PycparserNodeVisitor):
         node = node or self._root
         super(PycparserFinder, self).visit(node)
 
-    def visit_Enumerator(self, node):
+    def visit_Enum(self, node):
         if node.coord.file not in self.onlyin:
             return
         name = node.name
@@ -311,7 +377,8 @@ class PycparserFinder(astparsers.PycparserNodeVisitor):
 
 
 def pycparser_findall(filename, includes=(), defines=('XDRESS',), undefines=(),
-                      verbose=False, debug=False,  builddir='build'):
+                      extra_parser_args=(), verbose=False, debug=False, 
+                      builddir='build', language='c', clang_includes=()):
     """Automatically finds all API elements in a file via GCC-XML.
 
     Parameters
@@ -324,12 +391,17 @@ def pycparser_findall(filename, includes=(), defines=('XDRESS',), undefines=(),
         The list of extra macro definitions to apply.
     undefines : list of str, optional
         The list of extra macro undefinitions to apply.
+    extra_parser_args : list of str, optional
+        Further command line arguments to pass to the parser.
     verbose : bool, optional
         Flag to diplay extra information while describing the class.
     debug : bool, optional
         Flag to enable/disable debug mode.
     builddir : str, optional
         Location of -- often temporary -- build files.
+    language : str
+        Valid language flag.
+    clang_includes : ignored
 
     Returns
     -------
@@ -342,7 +414,8 @@ def pycparser_findall(filename, includes=(), defines=('XDRESS',), undefines=(),
 
     """
     root = astparsers.pycparser_parse(filename, includes=includes, defines=defines,
-                undefines=undefines, verbose=verbose, debug=debug, builddir=builddir)
+                undefines=undefines, extra_parser_args=extra_parser_args, 
+                verbose=verbose, debug=debug, builddir=builddir)
     basename = filename.rsplit('.', 1)[0]
     onlyin = set([filename, basename + '.h'])
     finder = PycparserFinder(root, onlyin=onlyin, verbose=verbose)
@@ -361,7 +434,8 @@ _finders = {
     }
 
 def findall(filename, includes=(), defines=('XDRESS',), undefines=(), 
-            parsers='gccxml', verbose=False, debug=False,  builddir='build'):
+            extra_parser_args=(), parsers='gccxml', verbose=False, debug=False, 
+            builddir='build', language='c++', clang_includes=()):
     """Automatically finds all API elements in a file.  This is the main entry point.
 
     Parameters
@@ -374,6 +448,8 @@ def findall(filename, includes=(), defines=('XDRESS',), undefines=(),
         The list of extra macro definitions to apply.
     undefines: list of str, optional
         The list of extra macro undefinitions to apply.
+    extra_parser_args : list of str, optional
+        Further command line arguments to pass to the parser.
     parsers : str, list, or dict, optional
         The parser / AST to use to use for the file.  Currently 'clang', 'gccxml', 
         and 'pycparser' are supported, though others may be implemented in the 
@@ -387,6 +463,10 @@ def findall(filename, includes=(), defines=('XDRESS',), undefines=(),
         Flag to enable/disable debug mode.
     builddir : str, optional
         Location of -- often temporary -- build files.
+    language : str
+        Valid language flag.
+    clang_includes : list of str, optional
+        clang-specific include paths.
 
     Returns
     -------
@@ -398,10 +478,11 @@ def findall(filename, includes=(), defines=('XDRESS',), undefines=(),
         A list of class names to wrap from the file.
 
     """
-    parser = astparsers.pick_parser(filename, parsers)
+    parser = astparsers.pick_parser(language, parsers)
     finder = _finders[parser]
     rtn = finder(filename, includes=includes, defines=defines, undefines=undefines, 
-                 verbose=verbose, debug=debug, builddir=builddir)
+                 extra_parser_args=extra_parser_args, verbose=verbose, debug=debug, 
+                 builddir=builddir, language=language, clang_includes=clang_includes)
     return rtn
 
 
@@ -505,24 +586,26 @@ class XDressPlugin(astparsers.ParserPlugin):
     def setup_basic(self, rc):
         """Does the easy part of setting up an autodecsibe environment"""
         # first pass -- gather and expand target
-        allsrc = set()
+        allsrc = {}
         varhasstar = False
         for i, var in enumerate(rc.variables):
             rc.variables[i] = var = ensure_apiname(var)
             if var.srcname == '*':
-                allsrc.add(var.srcfile)
+                allsrc.update(zip(var.srcfiles, [var.language]*len(var.srcfiles)))
                 varhasstar = True
         fnchasstar = False
         for i, fnc in enumerate(rc.functions):
             rc.functions[i] = fnc = ensure_apiname(fnc)
             if fnc.srcname == '*':
-                allsrc.add(fnc.srcfile)
+                allsrc.update(zip(fnc.srcfiles, [fnc.language]*len(fnc.srcfiles)))
+                #allsrc.update(fnc.srcfiles)
                 fnchasstar = True
         clshasstar = False
         for i, cls in enumerate(rc.classes):
             rc.classes[i] = cls = ensure_apiname(cls)
             if cls.srcname == '*':
-                allsrc.add(cls.srcfile)
+                allsrc.update(zip(cls.srcfiles, [cls.language]*len(cls.srcfiles)))
+                #allsrc.update(cls.srcfiles)
                 clshasstar = True
         self.allsrc = allsrc
         self.varhasstar = varhasstar
@@ -536,35 +619,30 @@ class XDressPlugin(astparsers.ParserPlugin):
             print("autoall: no API names to discover!")
             return
         allsrc = self.allsrc
+        kinds = ['variables', 'functions', 'classes']
 
         # second pass -- find all
-        allnames = {}
+        allfiles = {}
         cachefile = os.path.join(rc.builddir, 'autoname.cache')
         autonamecache = AutoNameCache(cachefile=cachefile)
-        for i, srcname in enumerate(allsrc):
-            srcfname, hdrfname, lang, ext = find_source(srcname, 
-                                                        sourcedir=rc.sourcedir)
-            filename = os.path.join(rc.sourcedir, srcfname)
-            print("autoall: searching {0} (from {1!r})".format(srcfname, srcname))
-            if autonamecache.isvalid(filename):
-                found = autonamecache[filename]
+        for i, (srcfile, lang) in enumerate(allsrc.items()):
+            print("autoall: searching {0}".format(srcfile))
+            if autonamecache.isvalid(srcfile):
+                found = autonamecache[srcfile]
             else:
-                found = findall(filename, includes=rc.includes, defines=rc.defines, 
-                                undefines=rc.undefines, parsers=rc.parsers, 
-                                verbose=rc.verbose, debug=rc.debug, 
-                                builddir=rc.builddir)
-                autonamecache[filename] = found
+                found = findall(srcfile, includes=rc.includes, defines=rc.defines, 
+                                undefines=rc.undefines, 
+                                extra_parser_args=rc.extra_parser_args, 
+                                parsers=rc.parsers, verbose=rc.verbose, 
+                                debug=rc.debug, builddir=rc.builddir, language=lang,
+                                clang_includes=rc.clang_includes)
+                autonamecache[srcfile] = found
                 autonamecache.dump()
-            allnames[srcname] = found
-            if 0 < len(found[0]):
-                fstr = ", ".join([str(_) for _ in found[0]])
-                print("autoall: found variables: " + fstr)
-            if 0 < len(found[1]):
-                fstr = ", ".join([str(_) for _ in found[1]])
-                print("autoall: found functions: " + fstr)
-            if 0 < len(found[2]):
-                fstr = ", ".join([str(_) for _ in found[2]])
-                print("autoall: found classes: " + fstr)
+            allfiles[srcfile] = found
+            for k, kind in enumerate(kinds):
+                if 0 < len(found[k]):
+                    fstr = ", ".join([str(_) for _ in found[k]])
+                    print("autoall: found {0}: {1}".format(kind, fstr))
             if 0 == i%rc.clear_parser_cache_period:
                 astparsers.clearmemo()
 
@@ -573,8 +651,9 @@ class XDressPlugin(astparsers.ParserPlugin):
             newvars = []
             for var in rc.variables:
                 if var.srcname == '*':
-                    for x in allnames[var.srcfile][0]:
-                        newvars.append(var._replace(srcname=x, tarname=x))
+                    for srcfile in var.srcfiles:
+                        for x in allfiles[srcfile][0]:
+                            newvars.append(var._replace(srcname=x, tarname=x))
                 else:
                     newvars.append(var)
             rc.variables = newvars
@@ -582,8 +661,9 @@ class XDressPlugin(astparsers.ParserPlugin):
             newfncs = []
             for fnc in rc.functions:
                 if fnc.srcname == '*':
-                    for x in allnames[fnc.srcfile][1]:
-                        newfncs.append(fnc._replace(srcname=x, tarname=x))
+                    for srcfile in fnc.srcfiles:
+                        for x in allfiles[srcfile][1]:
+                            newfncs.append(fnc._replace(srcname=x, tarname=x))
                 else:
                     newfncs.append(fnc)
             rc.functions = newfncs
@@ -591,8 +671,9 @@ class XDressPlugin(astparsers.ParserPlugin):
             newclss = []
             for cls in rc.classes:
                 if cls.srcname == '*':
-                    for x in allnames[cls.srcfile][2]:
-                        newclss.append(cls._replace(srcname=x, tarname=x))
+                    for srcfile in cls.srcfiles:
+                        for x in allfiles[srcfile][2]:
+                            newclss.append(cls._replace(srcname=x, tarname=x))
                 else:
                     newclss.append(cls)
             rc.classes = newclss
