@@ -166,6 +166,7 @@ import os
 import io
 import sys
 import argparse
+import warnings
 from pprint import pprint, pformat
 
 try:
@@ -174,7 +175,7 @@ except ImportError:
     argcomplete = None
 
 from .utils import NotSpecified, RunControl, DEFAULT_RC_FILE, DEFAULT_PLUGINS, \
-    exec_file
+    exec_file, parse_global_rc
 
 from .plugins import Plugins
 
@@ -184,6 +185,7 @@ if sys.version_info[0] >= 3:
 
 def main():
     """Entry point for xdress API generation."""
+    warnings.simplefilter('default')
     # Preprocess plugin names, which entails preprocessing the rc file
     preparser = argparse.ArgumentParser("XDress Pre-processor", add_help=False)
     preparser.add_argument('--rc', default=NotSpecified, 
@@ -213,11 +215,13 @@ def main():
         argcomplete.autocomplete(parser)
     ns = parser.parse_args()
     rc = plugins.merge_rcs()
+    rc._update(parse_global_rc())
     rc._update(rcdict)
     rc._update([(k, v) for k, v in ns.__dict__.items()])
     plugins.setup()
     plugins.execute()
     plugins.teardown()
+    plugins.exit()
 
 if __name__ == '__main__':
     main()
