@@ -1,4 +1,4 @@
-"""Helper functions for xdress.
+"""Helpers for xdress.
 
 :author: Anthony Scopatz <scopatz@gmail.com>
 
@@ -8,18 +8,29 @@ Utilities API
 from __future__ import print_function
 import os
 import io
+import re
+import ast
 import sys
+import glob
 import functools
 from copy import deepcopy
 from pprint import pformat
 from collections import Mapping, Iterable, Hashable, Sequence, namedtuple
 from hashlib import md5
 from warnings import warn
+
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
 
+try:
+    from enum import Enum, IntEnum
+except ImportError:
+    from ._enum import Enum, IntEnum
+
+import numpy as np
+
 if sys.version_info[0] >= 3:
     basestring = str
 
@@ -62,24 +73,68 @@ class indentstr(str):
         return getattr(super(indentstr, self), key)
 
 
+class Arg(IntEnum):
+    NONE = 0
+    TYPE = 1
+    LIT = 2
+    VAR = 3
+
+    def __str__(self):
+        return self.name
+
+    def __repr__(self):
+        return self.name
+
+
 def expand_default_args(methods):
     """This function takes a collection of method tuples and expands all of
     the default arguments, returning a set of all methods possible."""
     methitems = set()
-    for mkey, mrtn in methods:
+    for mkey, mval in methods:
         mname, margs = mkey[0], mkey[1:]
-        havedefaults = [3 == len(arg) for arg in margs]
+        mrtn = mval['return']
+        mdefargs = mval['defaults']
+        havedefaults = [arg is not Arg.NONE for arg in margs]
         if any(havedefaults):
             # expand default arguments
             n = havedefaults.index(True)
             items = [((mname,)+tuple(margs[:n]), mrtn)] + \
-                    [((mname,)+tuple(margs[:i]), mrtn) for i in range(n+1,len(margs)+1)]
+                [((mname,)+tuple(margs[:i]), mrtn) for i in range(n+1,len(margs)+1)]
             methitems.update(items)
         else:
             # no default args
             methitems.add((mkey, mrtn))
     return methitems
 
+_bool_literals = {'true': True, 'false': False}
+_c_literal_int = re.compile(r'^([+-]?([1-9][0-9]*|0o?[0-7]+|0x[0-9a-f]+|'
+                                   r'0b[01]+|0))u?(l{1,2})?u?$')
+_c_int_bases = {'0x': 16, '0o': 8, '0b': 2, '0': 8,
+                '00': 8, '01': 8, '02': 8, '03': 8, '04': 8, '05': 8, '06': 8, '07': 8}
+_c_float = re.compile(r'^([+-]?([0-9]+\.[0-9]*|\.[0-9]+)(e[+-]?[0-9]+)?)([lf]?)$')
+_c_float_types = {'': float, 'f': np.float32, 'l': np.longfloat}
+
+def c_literal(s):
+    """Convert a C/C++ literal to the corresponding Python value."""
+    s = s.strip()
+    try:
+        # ast.literal_eval isn't precisely correct, since there are Python
+        # literals which aren't valid C++, but it is close enough and faster
+        # than the custom code below in the common case.
+        return ast.literal_eval(s)
+    except (ValueError, SyntaxError):
+        pass
+    if s in _bool_literals:
+        return _bool_literals[s]
+    lo = s.lower()
+    m = _c_literal_int.match(lo)
+    if m:
+        base = _c_int_bases.get(m.group(2)[:2], 10)
+        return int(m.group(1), base)
+    m = _c_float.match(lo)
+    if m:
+        return _c_float_types[m.group(4)](m.group(1))
+    raise ValueError('unknown literal: {0!r}'.format(s))
 
 def newoverwrite(s, filename, verbose=False):
     """Useful for not forcing re-compiles and thus playing nicely with the
@@ -153,6 +208,10 @@ def ensuredirs(f):
     if not os.path.isdir(d):
         os.makedirs(d)
 
+def touch(filename):
+    """Opens a file and updates the mtime, like the posix command of the same name."""
+    with io.open(filename, 'a') as f:
+        os.utime(filename, None)
 
 def isvardesc(desc):
     """Tests if a description is a variable-type description."""
@@ -285,6 +344,24 @@ class RunControl(object):
                 v = self._updaters[k](getattr(self, k), v)
             setattr(self, k, v)
 
+def parse_global_rc():
+    '''Search a global xdressrc file and parse if it exists.
+    If nothing is found, an empty RunControl is returned.'''
+    home = os.path.expanduser('~')
+    globalrcs = [os.path.join(home, '.xdressrc'),
+                 os.path.join(home, '.xdressrc.py'),
+                 os.path.join(home, '.config', 'xdressrc'),
+                 os.path.join(home, '.config', 'xdressrc.py'),
+                 ]
+    rc = RunControl()
+    for globalrc in globalrcs:
+        if not os.path.isfile(globalrc):
+            continue
+        globalrcdict = {}
+        exec_file(globalrc, globalrcdict, globalrcdict)
+        rc._update(globalrcdict)
+    return rc
+
 _lang_exts = {
     'c': 'c',
     'c++': 'cpp',
@@ -335,11 +412,16 @@ def guess_language(filename, default='c++'):
     lang = _exts_lang.get(ext, default)
     return lang
 
-def find_source(basename, sourcedir='.'):
+def find_source(basename, sourcedir='.', language=None):
     """Finds a source filename, header filename, language name, and language
     source extension given a basename and source directory."""
-    files = os.listdir(sourcedir)
-    files = [f for f in files if f.startswith(basename + '.')]
+    if isinstance(basename, list):
+        files = [ os.path.abspath(x) for x in basename ]
+    elif os.path.isabs(basename):
+        files = [ basename ]
+    else:
+        files = os.listdir(sourcedir)
+        files = [f for f in files if f.startswith(basename + '.')]
     langs = dict([(f, guess_language(f, None)) for f in files])
     lang = src = hdr = srcext = None
     for f, l in langs.items():
@@ -354,9 +436,11 @@ def find_source(basename, sourcedir='.'):
         src = hdr
         lang = langs[hdr]
         srcext = _lang_exts[lang]
+    if language is not None and language is not NotSpecified:
+        lang = language
     return src, hdr, lang, srcext
 
-def find_filenames(srcname, tarname=None, sourcedir='src'):
+def find_filenames(srcname, tarname=None, sourcedir='src', language=None):
     """Returns a description dictionary for a class or function
     implemented in a source file and bound into a target file.
 
@@ -376,7 +460,8 @@ def find_filenames(srcname, tarname=None, sourcedir='src'):
 
     """
     desc = {}
-    srcfname, hdrfname, lang, ext = find_source(srcname, sourcedir=sourcedir)
+    srcfname, hdrfname, lang, ext = find_source(srcname, sourcedir=sourcedir,
+                                                language=language)
     desc['source_filename'] = srcfname
     desc['header_filename'] = hdrfname
     desc['language'] = lang
@@ -391,6 +476,38 @@ def find_filenames(srcname, tarname=None, sourcedir='src'):
         desc['srcpxd_filename'] = '{0}_{1}.pxd'.format(ext, tarname)
     return desc
 
+def extra_filenames(name):
+    """Returns a diictionary of extra filenames for an API element
+    implemented in a given language and bound into a target.
+
+    Parameters
+    ----------
+    name : apiname
+        The API element name.
+
+    Returns
+    -------
+    extra : dict
+        Extra filenames.
+
+    """
+    extra = {}
+    ext = _lang_exts[name.language]
+    tarbase = name.tarbase
+    if tarbase is None:
+        extra['pxd_filename'] = extra['pyx_filename'] = \
+                                extra['srcpxd_filename'] = None
+        srcbase = os.path.splitext(os.path.basename(name.srcfiles[0]))[0]
+        extra['pxd_base'] = srcbase
+        extra['cpppxd_base'] = '{0}_{1}'.format(ext, srcbase)
+    else:
+        extra['pxd_filename'] = '{0}.pxd'.format(tarbase)
+        extra['pyx_filename'] = '{0}.pyx'.format(tarbase)
+        extra['srcpxd_filename'] = '{0}_{1}.pxd'.format(ext, tarbase)
+        extra['pxd_base'] = tarbase
+        extra['cpppxd_base'] = '{0}_{1}'.format(ext, tarbase)
+    return extra
+
 def infer_format(filename, format):
     """Tries to figure out a file format."""
     if isinstance(format, basestring):
@@ -438,27 +555,37 @@ class DescriptionCache(object):
         else:
             self.cache = {}
 
-    def isvalid(self, name, filename, kind):
-        """Boolean on whether the cach value for a (name, filename, kind)
+    def _hash_srcfiles(self, srcfiles):
+        hashes = []
+        for srcfile in srcfiles:
+            with io.open(srcfile, 'rb') as f:
+                filebytes = f.read()
+            hashes.append(md5(filebytes).hexdigest())
+        return tuple(hashes)
+
+    def isvalid(self, name, kind):
+        """Boolean on whether the cach value for a (apiname, kind)
         tuple matches the state of the file on the system."""
-        key = (name, filename, kind)
+        key = tuple(name) + (kind,)
         if key not in self.cache:
             return False
-        cachehash = self.cache[key][0]
-        with io.open(filename, 'rb') as f:
-            filebytes = f.read()
-        currhash = md5(filebytes).hexdigest()
-        return cachehash == currhash
+        cachehashes = self.cache[key][0]
+        currhashes = self._hash_srcfiles(name.srcfiles)
+        return cachehashes == currhashes
 
     def __getitem__(self, key):
+        if len(key) == 2 and isinstance(key[0], apiname):
+            key = tuple(key[0]) + key[1:]
         return self.cache[key][1]  # return the description only
 
     def __setitem__(self, key, value):
-        name, filename, kind = key
-        with io.open(filename, 'rb') as f:
-            filebytes = f.read()
-        currhash = md5(filebytes).hexdigest()
-        self.cache[key] = (currhash, value)
+        if len(key) == 2 and isinstance(key[0], apiname):
+            name, kind = key
+            key = tuple(key[0]) + key[1:]
+        else:
+            name, kind = apiname(*key[0]), key[1]
+        currhashes = self._hash_srcfiles(name.srcfiles)
+        self.cache[key] = (currhashes, value)
 
     def __delitem__(self, key):
         del self.cache[key]
@@ -497,7 +624,7 @@ def merge_descriptions(descriptions):
             else:
                 desc[key] = deepcopy(value)
     # now sanitize methods
-    name = desc['name']
+    name = desc['name']['srcname'] # srcname or tarname?
     methods = desc.get('methods', {})
     for methkey, methval in list(methods.items()):
         if methval is None:
@@ -540,7 +667,7 @@ def parse_template(s, open_brace='<', close_brace='>', separator=','):
         return s
     if open_brace not in s and close_brace not in s:
         return s
-    t = [s.split(open_brace, 1)[0]]
+    t = [s.split(open_brace, 1)[0].split('::')[-1]]
     targs = split_template_args(s, open_brace=open_brace,
                                 close_brace=close_brace, separator=separator)
     for targ in targs:
@@ -549,41 +676,6 @@ def parse_template(s, open_brace='<', close_brace='>', separator=','):
     t.append(0)
     return tuple(t)
 
-#
-# API Name Tuples and Functions
-#
-
-apiname = namedtuple('apiname', ['srcname', 'srcfile', 'tarfile', 'tarname'])
-
-notspecified_apiname = apiname(*([NotSpecified]*len(apiname._fields)))
-
-def ensure_apiname(name):
-    """Takes user input and returns the corresponding apiname named tuple.
-    If the name is already an apiname instance with no NotSpecified fields,
-    this does not make a copy.
-    """
-    # ensure is a valid apiname
-    if isinstance(name, apiname):
-        pass
-    elif isinstance(name, Sequence):
-        name = notspecified_apiname._replace(**dict(zip(apiname._fields, name)))
-    elif isinstance(name, Mapping):
-        name = notspecified_apiname._replace(**name)
-
-    # ensure fields are not NotSpecified
-    updates = {}
-    if name.srcname is NotSpecified:
-        raise ValueError("apiname.srcname cannot be unspecified")
-    if name.srcfile is NotSpecified:
-        raise ValueError("apiname.srcfile cannot be unspecified")
-    if name.tarfile is NotSpecified:
-        updates['tarfile'] = name.srcfile
-    if name.tarname is NotSpecified:
-        updates['tarname'] = name.srcname
-    if 0 < len(updates):
-        name = name._replace(**updates)
-    return name
-
 #
 # Memoization
 #
@@ -596,7 +688,7 @@ def ishashable(x):
         elif isinstance(x, Iterable):
             return all(map(ishashable, x))
         else:
-         return True
+            return True
     else:
         return False
 
@@ -646,3 +738,182 @@ class memoize_method(object):
             return cache[key]
         else:
             return self.meth(*args, **kwargs)
+
+
+#
+# API Name Tuples and Functions
+#
+
+class apiname(namedtuple('apiname', ['srcname', 'srcfiles', 'tarbase', 'tarname',
+                                     'incfiles', 'sidecars', 'language'])):
+    """apiname(srcname=NotSpecified, srcfiles=NotSpecified, tarbase=NotSpecified,
+               tarname=NotSpecified, incfiles=NotSpecified, sidecars=NotSpecified,
+               language=NotSpecified)
+
+    This is a type, based on a named tuple, that represents the API element
+    names and is used to declare or discover where variables, functions, and
+    classes live for the rest of xdress.  All of the fields default to
+    NotSpecified if not given.  These feilds are the sames as what appear in
+    description dictionaries under the 'name' key.
+
+    Parameters
+    ----------
+    srcname : str, tuple, or NotSpecified
+        The element's API name in the original source code, eg. MyClass.
+    srcfiles : str, tuple of str, or NotSpecified
+        This is a sequence of unique strings which represents the file paths where
+        the API element *may* be defined. For example, ('myfile.c', 'myfile.h').
+        If the element is defined outside of these files, then the automatic
+        discovery or description may fail. When using ensure_apiname(), all strings
+        are automatically globbed, so 'src/myfile.*' will be converted to
+        ('src/myfile.c', 'src/myfile.h').  Since these files are parsed they must
+        actually exist on the filesystem.
+    tarbase : str or NotSpecified
+        The base portion of all automatically generated (target) files. This does
+        not include the directory or the file extension.  For example, if you
+        wanted cythongen to create a file name 'mynewfile.pyx' then the value here
+        would be simply 'mynewfile'.
+    tarname : str, tuple, or NotSpecified
+        The element's API name in the automatically generated (target) files,
+        e.g. MyNewClass.
+    incfiles : tuple of str or NotSpecified
+        This is a sequence of all files which must be #include'd to access the
+        srcname at compile time.  This should be as minimal of a set as possible,
+        preferably only one file.  For example even though most HDF5 API
+        declarations are split across multiple files, to use the public API you
+        only ever need to include 'hdf5.h'. Because these files are not parsed they
+        do not need to exist for xdress to run.  Thus the strings in incfiles are
+        not globbed.
+    sidecars : str, tuple of str, or NotSpecified
+        This is a sequence of all sidecar files to use for this API element. Like
+        srcfiles, these files must exist for xdress to run.  They similarly
+        globbed.  For example, 'myfile.py'.
+    language : str or NotSpecified
+        Flag for the language that the srcfiles are implemented in. Valid options
+        are: 'c', 'c++', 'f', 'fortran', 'f77', 'f90', 'python', and 'cython'.
+
+    See Also
+    --------
+    ensure_apiname
+
+    """
+    def __new__(cls, srcname=NotSpecified, srcfiles=NotSpecified,
+                tarbase=NotSpecified, tarname=NotSpecified, incfiles=NotSpecified,
+                sidecars=NotSpecified, language=NotSpecified):
+        return super(apiname, cls).__new__(cls, srcname, srcfiles, tarbase, tarname,
+                                           incfiles, sidecars, language)
+
+notspecified_apiname = apiname(*([NotSpecified]*len(apiname._fields)))
+
+def _ensure_srcfiles(inp):
+    """This ensures that srcsfiles is a tuple of filenames that has been
+    expanded out and the files actually exist on the file system.
+    """
+    if isinstance(inp, basestring):
+        inp = (inp,)
+    out = []
+    for f in inp:
+        if os.path.isfile(f):
+            out.append(f)
+        else:
+            out += sorted([x for x in glob.glob(f) if x not in out])
+    return tuple(out)
+
+def _guess_base(srcfiles, default=None):
+    """Guesses the base name for target files from source file names, or
+    failing that, a default value."""
+    basefiles = [os.path.basename(f) for f in srcfiles]
+    basename = os.path.splitext(os.path.commonprefix(basefiles))[0]
+    if len(basename) == 0:
+        basename = default
+    return basename
+
+def _guess_incfiles(srcfiles):
+    """This function guess potential include files from the headers that are
+    present in the source files.
+    """
+    incs = [s for s in srcfiles if os.path.splitext(s)[-1][1:] in _hdr_exts]
+    return tuple(incs)
+
+def _ensure_incfiles(inp):
+    """This ensures that incfiles is a tuple of unique filenames. This does not
+    test that the files actually exist on the file system nor glob the filenames.
+    """
+    if isinstance(inp, basestring):
+        inp = (inp,)
+    if inp is NotSpecified or inp is None:
+        inp = ()
+    out = []
+    for f in inp:
+        if not isinstance(f, basestring):
+            raise ValueError("{0!r} must be a string.".format(f))
+        if f not in out:
+            out.append(f)
+    return tuple(out)
+
+@memoize
+def find_sidecar(filename):
+    """Finds the sidecar for a filename, if it exists. Otherwise returns None."""
+    sc = os.path.splitext(filename)[0] + '.py'
+    sc = sc if os.path.isfile(sc) else None
+    return sc
+
+def _guess_sidecars(srcfiles):
+    """Gueses the sidecar file names from the source file names."""
+    scs = set(find_sidecar(f) for f in srcfiles)
+    scs.discard(None)
+    return tuple(sorted(scs))
+
+def _find_language(lang, srcfiles):
+    """Tries to discover the canonical language that the srcfiles are
+    implmenetd in."""
+    if isinstance(lang, basestring):
+        lang = lang.lower()
+        if lang not in _lang_exts:
+            raise ValueError('{0} is not a valid language'.format(lang))
+        return lang
+    langs = set(map(guess_language, srcfiles))
+    if len(langs) == 1:
+        return langs.pop()
+    for precedent in ['cython', 'c++', 'c', 'f90','f77', 'f', 'fortran', 'python']:
+        if precedent in langs:
+            return precedent
+    else:
+        raise ValueError("no valid language was found.")
+
+def ensure_apiname(name):
+    """Takes user input and returns the corresponding apiname named tuple.
+    If the name is already an apiname instance with no NotSpecified fields,
+    this does not make a copy.
+    """
+    # ensure is a valid apiname
+    if isinstance(name, apiname):
+        pass
+    elif isinstance(name, Sequence):
+        name = notspecified_apiname._replace(**dict(zip(apiname._fields, name)))
+    elif isinstance(name, Mapping):
+        name = apiname(**name)
+
+    # ensure fields are not NotSpecified
+    updates = {}
+    if name.srcname is NotSpecified:
+        raise ValueError("apiname.srcname cannot be unspecified")
+    if name.srcfiles is NotSpecified:
+        raise ValueError("apiname.srcfiles cannot be unspecified: {0}".format(name))
+    updates['srcfiles'] = _ensure_srcfiles(name.srcfiles)
+    if name.tarname is NotSpecified:
+        updates['tarname'] = name.srcname
+    if name.tarbase is NotSpecified:
+        updates['tarbase'] = _guess_base(updates['srcfiles'],
+                                         updates.get('tarname', name.tarname))
+    if name.incfiles is NotSpecified:
+        updates['incfiles'] = _guess_incfiles(updates['srcfiles'])
+    updates['incfiles'] = _ensure_incfiles(updates.get('incfiles', name.incfiles))
+    if name.sidecars is NotSpecified:
+        updates['sidecars'] = _guess_sidecars(updates['srcfiles'])
+    updates['sidecars'] = _ensure_srcfiles(updates.get('sidecars', name.sidecars))
+    if name.language not in _lang_exts:
+        updates['language'] = _find_language(name.language, updates['srcfiles'])
+    ensured = name._replace(**updates)
+    return name if name == ensured else ensured
+
